@@ -15,7 +15,8 @@ can be queried for individual datasets of interest.
 This module provides basic, low-level access to the RegTAP Registries using
 standardized TAP-based services.
 """
-from __future__ import print_function, division
+from __future__ import (
+    absolute_import, division, print_function, unicode_literals)
 import os
 from ..dal import scs, sia, ssa, sla, tap, query as dalq
 
@@ -32,7 +33,7 @@ _service_type_map = {
     "table": "tap"
 }
 
-def search(keywords=None, servicetype=None, waveband=None):
+def search(keywords=None, servicetype=None, waveband=None, datamodel=None):
     """
     execute a simple query to the RegTAP registry.
 
@@ -62,6 +63,12 @@ def search(keywords=None, servicetype=None, waveband=None):
        'euv',
        'x-ray'
        'gamma-ray'
+    datamodel : str
+        the name of the datamodel to search for; makes only sence in conjunction
+        with servicetype tap (or no servicetype).
+
+        See http://wiki.ivoa.net/twiki/bin/view/IVOA/IvoaDataModel for more
+        informations about data models.
 
     Returns
     -------
@@ -72,12 +79,11 @@ def search(keywords=None, servicetype=None, waveband=None):
     --------
     RegistryResults
     """
-    if not any((keywords, servicetype, waveband)):
+    if not any((keywords, servicetype, waveband, datamodel)):
         raise dalq.DALQueryError(
             "No search parameters passed to registry search")
 
-    joins = set(["rr.interface"])
-    joins = set(["rr.resource"])
+    joins = set(["rr.interface", "rr.resource"])
     wheres = list()
 
     if keywords:
@@ -112,6 +118,16 @@ def search(keywords=None, servicetype=None, waveband=None):
         wheres.append("1 = ivo_hashlist_has('{}', waveband)".format(
             tap.escape(waveband)))
 
+    if datamodel:
+        joins.add("rr.interface")
+        joins.add("rr.res_detail")
+
+        wheres.append("intf_type = 'vs:paramhttp'")
+        wheres.append("detail_xpath='/capability/dataModel/@ivo-id'")
+        wheres.append(
+            "1=ivo_nocasematch(detail_value, 'ivo://ivoa.net/std/{0}%')".format(
+                tap.escape(datamodel)))
+
     query = """SELECT DISTINCT rr.interface.*, rr.capability.*, rr.resource.*
     FROM rr.capability
     {}
@@ -122,24 +138,33 @@ def search(keywords=None, servicetype=None, waveband=None):
     )
 
     service = tap.TAPService(REGISTRY_BASEURL)
-    query = tap.TAPQuery(service.baseurl, query, maxrec=service.hardlimit)
-    query.RESULTS_CLASS = RegistryResults
+    query = RegistryQuery(service.baseurl, query, maxrec=service.hardlimit)
     return query.execute()
 
 
-class RegistryResults(tap.TAPResults):
+class RegistryQuery(tap.TAPQuery):
+    def execute(self):
+        """
+        submit the query and return the results as a RegistryResults instance
+
+        Raises
+        ------
+        DALServiceError
+           for errors connecting to or communicating with the service
+        DALQueryError
+           for errors either in the input query syntax or
+           other user errors detected by the service
+        DALFormatError
+           for errors parsing the VOTable response
+        """
+        return RegistryResults(self.execute_votable(), self.queryurl)
+
+
+class RegistryResults(dalq.DALResults):
     """
     an iterable set of results from a registry query. Each record is
-    returned as RegistryResultse
+    returned as RegistryResults
     """
-    def __init__(self, votable, url=None, version="1.0"):
-        """
-        initialize the results.  This constructor is not typically called
-        by directly applications; rather an instance is obtained from calling
-        a SIAQuery's execute().
-        """
-        super(RegistryResults, self).__init__(votable, url, "regtap", version)
-
     def getrecord(self, index):
         """
         return all the attributes of a resource record with the given index
@@ -174,33 +199,36 @@ class RegistryResource(dalq.Record):
     accessurl : str
        when the resource is a service, the service's access URL.
     """
+
+    _service = None
+
     @property
     def ivoid(self):
         """
         the IVOA identifier for the resource.
         """
-        return self.get("ivoid")
+        return self.get("ivoid", decode=True)
 
     @property
     def res_type(self):
         """
         the resource types that characterize this resource.
         """
-        return self.get("res_type")
+        return self.get("res_type", decode=True)
 
     @property
     def short_name(self):
         """
         the short name for the resource
         """
-        return self.get("short_name")
+        return self.get("short_name", decode=True)
 
     @property
     def res_title(self):
         """
         the title of the resource
         """
-        return self.get("res_title", None)
+        return self.get("res_title", default=None, decode=True)
 
     @property
     def content_levels(self):
@@ -208,7 +236,7 @@ class RegistryResource(dalq.Record):
         a list of content level labels that describe the intended audience
         for this resource.
         """
-        return self.get("content_level", "").split("#")
+        return self.get("content_level", default="", decode=True).split("#")
 
     @property
     def res_description(self):
@@ -219,14 +247,14 @@ class RegistryResource(dalq.Record):
         --------
         SimpleResource.describe
         """
-        return self.get("res_description")
+        return self.get("res_description", decode=True)
 
     @property
     def reference_url(self):
         """
         URL pointing to a human-readable document describing this resource.
         """
-        return self.get("reference_url")
+        return self.get("reference_url", decode=True)
 
     @property
     def creators(self):
@@ -234,21 +262,21 @@ class RegistryResource(dalq.Record):
         The creator(s) of the resource
         in the ordergiven by the resource record author
         """
-        return self.get("creator_seq", "").split(";")
+        return self.get("creator_seq", default="", decode=True).split(";")
 
     @property
     def content_types(self):
         """
         list of natures or genres of the content of the resource.
         """
-        return self.get("content_type").split("#")
+        return self.get("content_type", decode=True).split("#")
 
     @property
     def source_format(self):
         """
         The format of source_value.
         """
-        return self.get("source_format")
+        return self.get("source_format", decode=True)
 
     @property
     def region_of_regard(self):
@@ -257,28 +285,28 @@ class RegistryResource(dalq.Record):
         by which a positional query against this resource should be "blurred"
         in order to get an appropriate match.
         """
-        return float(self.get("region_of_regard", "0"))
+        return float(self.get("region_of_regard", 0))
 
     @property
     def waveband(self):
         """
         a list of names of the wavebands that the resource provides data for
         """
-        return self.get("waveband", "").split("#")
+        return self.get("waveband", default="", decode=True).split("#")
 
     @property
     def access_url(self):
         """
         the URL that can be used to access the service resource.
         """
-        return self.get("access_url")
+        return self.get("access_url", decode=True)
 
     @property
     def standard_id(self):
         """
         the IVOA standard identifier
         """
-        return self.get("standard_id")
+        return self.get("standard_id", decode=True)
 
     @property
     def service(self):
@@ -288,20 +316,18 @@ class RegistryResource(dalq.Record):
         not a recognized DAL service.  Currently, only Conesearch, SIA, SSA,
         and SLA services are supported.
         """
-        if not self.access_url:
-            return None
+        if self.access_url:
+            for key, value in {
+                "ivo://ivoa.net/std/conesearch":  scs.SCSService,
+                "ivo://ivoa.net/std/sia":  sia.SIAService,
+                "ivo://ivoa.net/std/ssa":  ssa.SSAService,
+                "ivo://ivoa.net/std/sla":  sla.SLAService,
+                "ivo://ivoa.net/std/tap":  tap.TAPService,
+            }.items():
+                if key in self.standard_id:
+                    self._service = value(self.access_url)
 
-        for key, value in {
-            "ivo://ivoa.net/std/conesearch":  scs.SCSService,
-            "ivo://ivoa.net/std/sia":  sia.SIAService,
-            "ivo://ivoa.net/std/ssa":  ssa.SSAService,
-            "ivo://ivoa.net/std/sla":  sla.SLAService,
-            "ivo://ivoa.net/std/tap":  tap.TAPService,
-        }.items():
-            if self.standard_id in key:
-                return value(self.access_url)
-
-        return None
+        return self._service
 
     def search(self, *args, **keys):
         """
@@ -332,7 +358,7 @@ class RegistryResource(dalq.Record):
         if not self.service:
             raise dalq.DALServiceError(
                 "resource, {0}, is not a searchable service".format(
-                    self.shortname))
+                    self.short_name))
 
         return self.service.search(*args, **keys)
 
