@@ -1,11 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-import dateutil.parser
+from astropy.time import Time
 from . import plainxml
 
 class _JobParser(plainxml.StartEndHandler):
 	def __init__(self):
 		plainxml.StartEndHandler.__init__(self)
-		self.job = {}
+		self.job = {
+			"version": "1.0"
+		}
+
+	def _start_job(self, name, attrs):
+		if "version" in attrs:
+			self.job["version"] = attrs["version"]
 
 	def _end_jobId(self, name, attrs, content):
 		self.job["jobId"] = content.strip()
@@ -29,7 +35,9 @@ class _JobParser(plainxml.StartEndHandler):
 		self.job["executionDuration"] = content.strip()
 
 	def _end_destruction(self, name, attrs, content):
-		self.job["destruction"] = dateutil.parser.parse(content.strip())
+		time = Time(
+			content.strip(), format="isot")
+		self.job["destruction"] = time.datetime
 
 	def _end_message(self, name, attrs, content):
 		self.job["message"] = content.strip()
